#include "includes.h"

#define WIDTH 800
#define HEIGHT 600

//mouse speed 1-20
#define INITIAL_MOUSE_SPEED 10
//move speed 1-20
#define INITIAL_MOVE_SPEED  10

//light parameters
float ambientLight[] = { 0.2f, 0.2f, 0.2f, 1.0f };
float diffuseLight[] = { 0.4f, 0.4f, 0.4f, 1.0f };
float positionLight0[] = { 200.0f,  100.0f, 100.0f, 1.0f };
float positionLight1[] = { 50.0f,  100.0f, -100.0f, 1.0f };
float positionLight2[] = { -100.0f, -50.0f, 0.0f, 1.0f };

//scene parameters
model_t scene;
float ratio; 				// width to height ratio
float x=0.0f,y=0.0f,z=0.0f; 		//current position
float lx=0.0f,ly=0.0f,lz=-1.0f;		//direction I look at
int h = HEIGHT, w = WIDTH;		//width and height of the scene
int middlex = WIDTH/2, middley= HEIGHT/2;	//middle of the scene
static GLint snowman_display_list;	//display list
static GLint scene_display_list;	//display list
char pos_txt[40];

//text displaying parameters
int font=(int)GLUT_BITMAP_8_BY_13;
int bitmapHeight=13;

//keyboard movement variables
int deltaMove = 0, deltaStrafe = 0;
int move_speed = INITIAL_MOVE_SPEED;

//mouse movement variables
int mouseReferenceX = WIDTH/2, mouseReferenceY = HEIGHT/2;
float mouseRotX = 0.0, mouseRotY = 0.0;
float angleLR=0.0, angleUD = 0.0;
int mouse_speed = INITIAL_MOUSE_SPEED;

//FPS
int frame,time,timebase=0;
char fps[30];


void changeSize(int w1, int h1)
	{

	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h1 == 0)
		h1 = 1;

	w = w1;
	h = h1;
	middlex = w/2; middley= h/2;
	ratio = 1.0f * w / h;
	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, w, h);

	// Set the clipping volume
	gluPerspective(45,ratio,0.1,1000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(x, y, z, 
		      x + lx,y + ly,z + lz,
			  0.0f,1.0f,0.0f);
	}

void setMousePositionReferencePoint(int x, int y) {
    mouseReferenceX = x;
    mouseReferenceY = y;
}


void drawSnowMan() {


	glColor3f(1.0f, 1.0f, 1.0f);

// Draw Body	
	glTranslatef(0.0f ,0.75f, 0.0f);
	glutSolidSphere(0.75f,20,20);


// Draw Head
	glTranslatef(0.0f, 1.0f, 0.0f);
	glutSolidSphere(0.25f,20,20);

// Draw Eyes
	glPushMatrix();
	glColor3f(0.0f,0.0f,0.0f);
	glTranslatef(0.05f, 0.10f, 0.18f);
	glutSolidSphere(0.05f,10,10);
	glTranslatef(-0.1f, 0.0f, 0.0f);
	glutSolidSphere(0.05f,10,10);
	glPopMatrix();

// Draw Nose
	glColor3f(1.0f, 0.5f , 0.5f);
	glRotatef(0.0f,1.0f, 0.0f, 0.0f);
	glutSolidCone(0.08f,0.5f,10,2);
}



GLuint createSnowmanDL() {

    	int i, j;
	GLuint snowManDL;
	vector_t normal_v;

	// Create the id for the list
	snowManDL = glGenLists(2);

	glNewList(snowManDL+1,GL_COMPILE);
		drawSnowMan();
	glEndList();
	// start list
	glNewList(snowManDL,GL_COMPILE);

	// call the function that contains the rendering commands
	for(i = -3; i < 3; i++)
		for(j=-3; j < 3; j++) {
			glPushMatrix();
			glTranslatef(i*10.0,0,j * 10.0);
			glCallList(snowManDL+1);
			glPopMatrix();
		}

	glBegin(GL_QUADS);
		glColor3f(0.8f, 0.8f, 0.8f);

		// Back Face
		normal_f(4.0f,  4.0f,  4.0f, 4.0f,  6.0f,  4.0f, 6.0f,  6.0f,  4.0f, &normal_v);
		glNormal3f(normal_v.x, normal_v.y, normal_v.z);
		glVertex3f( 4.0f,  4.0f,  4.0f);	// Point 1 (Back)
		glVertex3f( 4.0f,  6.0f,  4.0f);	// Point 2 (Back)
		glVertex3f( 6.0f,  6.0f,  4.0f);	// Point 3 (Back)
		glVertex3f( 6.0f,  4.0f,  4.0f);	// Point 4 (Back)
		// Top Face
		normal_f (4.0f,  6.0f,  4.0f, 4.0f,  6.0f,  6.0f,  6.0f,  6.0f,  6.0f, &normal_v);
		glNormal3f(normal_v.x, normal_v.y, normal_v.z);
		glVertex3f( 4.0f,  6.0f,  4.0f);	// Point 1 (Top)
		glVertex3f( 4.0f,  6.0f,  6.0f);	// Point 2 (Top)
		glVertex3f( 6.0f,  6.0f,  6.0f);	// Point 3 (Top)
		glVertex3f( 6.0f,  6.0f,  4.0f);	// Point 4 (Top)
		// Bottom Face
		normal_f ( 4.0f,  4.0f,  4.0f, 6.0f,  4.0f,  4.0f, 6.0f,  4.0f,  6.0f,  &normal_v);
                glNormal3f(normal_v.x, normal_v.y, normal_v.z);

		glVertex3f( 4.0f,  4.0f,  4.0f);	// Point 1 (Bottom)
		glVertex3f( 6.0f,  4.0f,  4.0f);	// Point 2 (Bottom)
		glVertex3f( 6.0f,  4.0f,  6.0f);	// Point 3 (Bottom)
		glVertex3f( 4.0f,  4.0f,  6.0f);	// Point 4 (Bottom)
	glEnd();

	
	// endList
	glEndList();

	return(snowManDL);
}

void initScene() {

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHT2);

	// Assign created components to GL_LIGHT0
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT0, GL_POSITION, positionLight0);

	// Assign created components to GL_LIGHT1
	glLightfv(GL_LIGHT1, GL_AMBIENT, ambientLight);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT1, GL_POSITION, positionLight1);

	// Assign created components to GL_LIGHT2
	glLightfv(GL_LIGHT2, GL_AMBIENT, ambientLight);
	glLightfv(GL_LIGHT2, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT2, GL_POSITION, positionLight2);
	
	glShadeModel(GL_FLAT);
	//glShadeModel(GL_SMOOTH);

	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

	snowman_display_list = createSnowmanDL();
}

void orientMe(float angLR, float angUD) {
	lx = sin(angLR);
	lz = -cos(angLR);
	ly = -sin(angUD);
	glLoadIdentity();
	gluLookAt(x, y, z, 
		      x + lx,y + ly,z + lz,
			  0.0f,1.0f,0.0f);
}


void moveMe(int move, int strafe) {
	x = x + move*(lx)*0.01*move_speed - strafe*(lz)*0.005*move_speed;
	z = z + move*(lz)*0.01*move_speed + strafe*(lx)*0.005*move_speed;
	y = y + move*(ly)*0.01*move_speed;
	glLoadIdentity();
	gluLookAt(x, y, z, 
		      x + lx,y + ly,z + lz,
			  0.0f,1.0f,0.0f);
}



void setOrthographicProjection() {

	// switch to projection mode
	glMatrixMode(GL_PROJECTION);
	// save previous matrix which contains the 
	//settings for the perspective projection
	glPushMatrix();
	// reset matrix
	glLoadIdentity();
	// set a 2D orthographic projection
	gluOrtho2D(0, w, 0, h);
	// invert the y axis, down is positive
	glScalef(1, -1, 1);
	// mover the origin from the bottom left corner
	// to the upper left corner
	glTranslatef(0, -h, 0);
	glMatrixMode(GL_MODELVIEW);
}

void resetPerspectiveProjection() {
	// set the current matrix to GL_PROJECTION
	glMatrixMode(GL_PROJECTION);
	// restore previous settings
	glPopMatrix();
	// get back to GL_MODELVIEW matrix
	glMatrixMode(GL_MODELVIEW);
}

void renderBitmapString(float x, float y, void *font,char *string)
{
  
  char *c;
  // set position to start drawing fonts
  glRasterPos2f(x, y);
  // loop all the characters in the string
  for (c=string; *c != '\0'; c++) {
    glutBitmapCharacter(font, *c);
  }

}

void renderScene(void) {

	angleLR += mouseRotX;
	mouseRotX = 0;
	angleUD += mouseRotY;
	if (angleUD >  M_PI_2) angleUD =  M_PI_2;
	if (angleUD < -M_PI_2) angleUD = -M_PI_2;
	mouseRotY = 0;
	orientMe(angleLR, angleUD);
    	
	if (deltaMove || deltaStrafe)
		moveMe(deltaMove, deltaStrafe);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glLightfv(GL_LIGHT0, GL_POSITION, positionLight0);
	glLightfv(GL_LIGHT1, GL_POSITION, positionLight1);

// Draw ground

//	glColor3f(0.9f, 0.9f, 0.9f);
//	glBegin(GL_QUADS);
//		glVertex3f(-100.0f, 0.0f, -100.0f);
//		glVertex3f(-100.0f, 0.0f,  100.0f);
//		glVertex3f( 100.0f, 0.0f,  100.0f);
//		glVertex3f( 100.0f, 0.0f, -100.0f);
//	glEnd();

// Draw 36 SnowMen
	//glCallList(snowman_display_list);

	DrawModel_f(&scene, 0, 0, 0, 0, 0, 0);
	
	frame++;
	time=glutGet(GLUT_ELAPSED_TIME);
	
	if (time - timebase > 100) {
		sprintf(fps,"FPS:%4.2f",frame*1000.0/(time-timebase));
		timebase = time;		
		frame = 0;
	}

	glColor3f(1.0f,0.0f,0.0f);
	setOrthographicProjection();
	glPushMatrix();
	glLoadIdentity();

	glDisable(GL_LIGHTING);
	renderBitmapString(30,15,(void *)font,fps);
	
	sprintf(pos_txt, "X = %4.2f, Y = %4.2f, Z = %4.2f", x, z, y);
	renderBitmapString(150,15,(void *)font,pos_txt);
	
	sprintf(pos_txt, "angle LR = %4d deg, angle UD = %3d deg", ((int)(angleLR/M_PI*180))%360, -((int)(angleUD/M_PI*180))%360);
	renderBitmapString(450,15,(void *)font,pos_txt);

	sprintf(pos_txt, "Move Speed = %2d, Rotate Speed = %2d", move_speed, mouse_speed);
	renderBitmapString(150,30,(void *)font,pos_txt);

	renderBitmapString(30,35,(void *)font,"Esc - Quit");
	glEnable(GL_LIGHTING);

	glPopMatrix();
	resetPerspectiveProjection();

	glutSwapBuffers();
}

void processNormalKeys(unsigned char key, int x, int y) {

	if (key == 27) 
		exit(0);
	if (key == '[' && mouse_speed > 1)
		mouse_speed--;
	if (key == ']' && mouse_speed < 20)
		mouse_speed++;
	if (key == ',' && move_speed > 1)
		move_speed--;
	if (key == '.' && move_speed < 20)
		move_speed++;
}

void pressKey(int key, int x, int y) {

	switch (key) {
		case GLUT_KEY_LEFT : deltaStrafe = -1; break;
		case GLUT_KEY_RIGHT : deltaStrafe = 1; break;
		case GLUT_KEY_UP : deltaMove = 1; break;
		case GLUT_KEY_DOWN : deltaMove = -1; break;
	}

}

void releaseKey(int key, int x, int y) {
	switch (key) {
	    case GLUT_KEY_LEFT:   if (deltaStrafe < 0) 
				      deltaStrafe = 0;
				  break;
	    case GLUT_KEY_RIGHT:  if (deltaStrafe > 0) 
				      deltaStrafe = 0;
				  break;
	    case GLUT_KEY_UP:     if (deltaMove > 0) 
				      deltaMove = 0;
				  break;
	    case GLUT_KEY_DOWN:   if (deltaMove < 0) 
				      deltaMove = 0;
				  break;
	}
}

void initWindow() {
	glutIgnoreKeyRepeat(1);
	glutKeyboardFunc(processNormalKeys);
	glutSpecialFunc(pressKey);
	glutSpecialUpFunc(releaseKey);
	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);
	glutReshapeFunc(changeSize);
	initScene();

}

void processMousePassiveMotion(int x, int y) {
	mouseRotX = (x - mouseReferenceX) / 2000.0 * mouse_speed;
	mouseRotY = (y - mouseReferenceY) / 2000.0 * mouse_speed;
	setMousePositionReferencePoint(x, y);
	
	if (x < 50 || x > w - 50 || y < 50 || y > h - 50) {
	    glutWarpPointer(middlex,middley);
	    setMousePositionReferencePoint(middlex, middley);
	}
}


int main(int argc, char **argv)
{
    	
	if (argc != 2) {
		 printf ("\nWrong number of parameters!\n\nUsage:\n\t ./FPP_3Dmap input_file\n");
	         exit(-1);
	}
	
	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(WIDTH,HEIGHT);
	glutCreateWindow("First Person 3D Map View");

	// register all callbacks
	initWindow();
	glutPassiveMotionFunc(processMousePassiveMotion);
	glutWarpPointer(middlex,middley);
	setMousePositionReferencePoint(middlex, middley);
	
	// hide mouse cursor
	glutSetCursor(GLUT_CURSOR_NONE);

	LoadModel(argv[1], &scene);

	glutMainLoop();

	return(0);
}
