#include "includes.h"

float getParamValue (char *param_name, FILE **file)
{
        float f = 0;
	char *line;
	char format[128];
	size_t len = 0;
	ssize_t read;

	strcpy (&(format[0]), param_name);
	strcat (&(format[0]), " = %f");
	
	//go to the beggining of file
	if (fseek(*file, 0, SEEK_SET) != 0) {
        	printf ("getParamValue: fseek failed.\n");
		exit(-1);
	}

	while ((read = getline(&line, &len, (*file))) != -1) {
		if (sscanf(line, format, &f) != 0)
			return f;
        }
	
	printf ("getParamValue: could not get the value of \"%s\"\n", param_name);
	exit(-1);
}

map_t getMapFromFile (FILE **file) 
{
	map_t mapa;
	char *line;
	size_t len = 0;
	ssize_t read;
	int found_flag = 0;
	int i=0, j=0;
	char ch;
	
	//go to the beggining of file
        if (fseek(*file, 0, SEEK_SET) != 0) {
                printf ("getParamValue: fseek failed.\n");
                exit(-1);
        }

	while ((read = getline(&line, &len, (*file))) != -1) {
                if (strcmp(line, "Map:\n") == 0) {
			found_flag = 1;
                        break;
		}
        }

	if (found_flag == 0) {
	    printf ("getMapFromFile: could not get the map object in input file\n");
	    exit(-1);
	}
	
	//clear map first
	for (i=0; i <= 127; i++)
	    for (j=0; j <= 127; j++)
		mapa.cells[i][j] = ' ';
	
	i=0; j=0;
	
	while (1) {
		ch = fgetc(*file);
		
		if (ch == EOF) break;
		if (ch == '\n') {i++; j=0; continue;}
		mapa.cells[i][j++] = ch;
	}
	
	return mapa;
}

void writePolygon (FILE **file, vertex_t v1, vertex_t v2, vertex_t v3) {
	fprintf (*file, "%f , %f , %f , %f , %f , %f , %f , %f , %f \n",
		v1.x, v1.y, v1.z, v2.x, v2.y, v2.z, v3.x, v3.y, v3.z);
}

vertex_t distortVertex(vertex_t v, float max_dist)
{
        vertex_t v_dist;
        int rand_val;

	rand_val = (int) -(201.0*rand()/(RAND_MAX+1)) - 100; //gerenate a random value in <-100, 100>
	v_dist.x = v.x + max_dist*rand_val/100;

	rand_val = (int) -(201.0*rand()/(RAND_MAX+1)) - 100; //gerenate a random value in <-100, 100>
	v_dist.y = v.y + max_dist*rand_val/100;

	rand_val = (int) -(201.0*rand()/(RAND_MAX+1)) - 100; //gerenate a random value in <-100, 100>
	v_dist.z = v.z + max_dist*rand_val/100;

	return v_dist;
}


void generateMap (char *input_file, char *output_file)
{

    	int i, j, k, l, m;
	FILE *file = fopen(input_file, "r");

	if (file == NULL) {
	    	printf ("generateMap: could not open input file.\n");
		exit(-1);
	}
	
	float ceiling = getParamValue ("Ceiling", &file);
	float height = getParamValue ("Wall height", &file);
	float width = getParamValue ("Wall width", &file);
	int lpr_v = (int)getParamValue ("Vertical lines per wall", &file);
	int lpr_h = (int)getParamValue ("Horizontal lines per wall", &file);
	float max_distort_percent = getParamValue ("Max vertex distortion (0-99)", &file);
	map_t mapa = getMapFromFile(&file);

	float skok_v = (float)1 / (lpr_v + 1);
	float skok_h = (float)1 / (lpr_h + 1);
	float max_distort_val = (skok_v < skok_h) ? skok_v : skok_h;
	max_distort_val = max_distort_val * max_distort_percent / 100;
	
	long number_of_polygons = 0;

	vertex_t *vertexList;
	
	int k_start, k_stop, l_start, l_stop, nr;
	
	vertex_t v1, v2, v3;

	fclose (file);
	
	file = fopen(output_file, "w");

	fprintf(file, "                                                               \n");

	for (i=0; i <= 127; i++) {
        	for (j=0; j <= 127; j++) {
			
			if (mapa.cells[i][j] == '-') { //sciana pozioma
				
				k_start = 1;  k_stop = lpr_v + 1;
				
				if (j != 0 && mapa.cells[i][j-1] == '|')  k_start = 1 - (lpr_v + 1)/2;
				if (j != 127 && mapa.cells[i][j+1] == '|')  k_stop = (int) ((lpr_v + 1) * 1.5);

				if ((i != 127 && mapa.cells[i+1][j] == '|') || (i != 0 && mapa.cells[i-1][j] == '|')) {
					if (j != 0 && mapa.cells[i][j-1] != '-'	&& j != 127 && mapa.cells[i][j+1] != '-') continue;
					if (j != 0 && mapa.cells[i][j-1] != '-' && j != 127 && mapa.cells[i][j+1] == '-') 
						k_start = (int) 1 + (lpr_v + 1)/2;
					if (j != 0 && mapa.cells[i][j-1] == '-' && j != 127 && mapa.cells[i][j+1] != '-')
						k_stop = (int) (lpr_v + 1)/2;

				}
				
				vertexList = (vertex_t *) malloc(((k_stop - k_start + 2) * (lpr_h + 2)) * sizeof(vertex_t));
				nr = 0;

				//generate all vertices for this wall
				for (l = 1; l <= lpr_h + 2; l++) 
					for (k = k_start; k <= k_stop+1; k++) {
						vertexList[nr].x = width * (j + (float)(k-1)/(lpr_v+1)) ;
						vertexList[nr].y = height * ((float)(lpr_h + 2 - l) / (lpr_h + 1));
						vertexList[nr].z = width * (i + 0.5);
						if (l != 1 && l != lpr_h + 2 && k != k_start && k != k_stop +1)
							vertexList[nr] = distortVertex(vertexList[nr], max_distort_val);
						//printf("x=%f, y=%f, z=%f\n", vertexList[nr].x, vertexList[nr].y, vertexList[nr].z);
						nr++;
					}

				//create polygons from generated vertices
				for (k = 1; k <= k_stop - k_start + 1; k++)
					for (l = 1; l <= lpr_h + 1; l++) {
						v1 = vertexList[(l - 1) * (k_stop - k_start + 2) + k - 1];
						v2 = vertexList[(l - 1) * (k_stop - k_start + 2) + k    ];
						v3 = vertexList[ l      * (k_stop - k_start + 2) + k - 1];

						writePolygon(&file, v1, v2, v3); 
						number_of_polygons++;
						
						v1 = v3;
					 	v3 = vertexList[ l      * (k_stop - k_start + 2) + k    ];
					 
						writePolygon(&file, v1, v2, v3); 
						number_of_polygons++;	
					}

				free (vertexList);
			}
			if (mapa.cells[i][j] == '|') { //sciana pionowa

				k_start = 1;  k_stop = lpr_v + 1;
				
				if (i != 0 && mapa.cells[i-1][j] == '-')  k_start = 1 - (lpr_v + 1)/2;
				if (i != 127 && mapa.cells[i+1][j] == '-')  k_stop = (int) ((lpr_v + 1) * 1.5);
				
				if ((j != 127 && mapa.cells[i][j+1] == '-') || (j != 0 && mapa.cells[i][j-1] == '-')) {
					if (i != 0 && mapa.cells[i-1][j] != '|'	&& i != 127 && mapa.cells[i+1][j] != '|') continue;
					if (i != 0 && mapa.cells[i-1][j] != '|' && i != 127 && mapa.cells[i+1][j] == '|') 
						k_start = (int) 1 + (lpr_v + 1)/2;
					if (i != 0 && mapa.cells[i-1][j] == '|' && i != 127 && mapa.cells[i+1][j] != '|')
						k_stop = (int) (lpr_v + 1)/2;

				}
	
				vertexList = (vertex_t *) malloc(((k_stop - k_start + 2) * (lpr_h + 2)) * sizeof(vertex_t));
                                nr = 0;
				
				//generate all vertices for this wall
				for (l = 1; l <= lpr_h + 2; l++) 
					for (k = k_start; k <= k_stop+1; k++) {
						vertexList[nr].z = width * (i + (float)(k-1)/(lpr_v+1)) ;
						vertexList[nr].y = height * ((float)(lpr_h + 2 - l) / (lpr_h + 1));
						vertexList[nr].x = width * (j + 0.5);
						if (l != 1 && l != lpr_h + 2 && k != k_start && k != k_stop +1)
							vertexList[nr] = distortVertex(vertexList[nr], max_distort_val);
						//printf("x=%f, y=%f, z=%f\n", vertexList[nr].x, vertexList[nr].y, vertexList[nr].z);
						nr++;
					}

				//create polugons from generated vertices
				for (k = 1; k <= k_stop - k_start + 1; k++)
					for (l = 1; l <= lpr_h + 1; l++) {
						v1 = vertexList[(l - 1) * (k_stop - k_start + 2) + k - 1];
						v2 = vertexList[(l - 1) * (k_stop - k_start + 2) + k    ];
						v3 = vertexList[ l      * (k_stop - k_start + 2) + k - 1];

						writePolygon(&file, v1, v2, v3); 
						number_of_polygons++;
						
						v1 = v3;
					 	v3 = vertexList[ l      * (k_stop - k_start + 2) + k    ];
					 
						writePolygon(&file, v1, v2, v3); 
						number_of_polygons++;	
					}

				free (vertexList);
			}
			
			if (mapa.cells[i][j] == 'c' && ceiling == 1) { //podoga / sufit
				
				k_start = 1;  k_stop = lpr_v + 1;
				l_start = 1;  l_stop = lpr_v + 2;
				
				if (j != 0 && mapa.cells[i][j-1] == '|' || (j != 0 && mapa.cells[i][j-1] == '-' && 
				   (i != 0 && mapa.cells[i-1][j-1] == '|' || i != 127 && mapa.cells[i+1][j-1] == '|')))  
					k_start = 1 - (lpr_v + 1)/2;
				
				if (j != 127 && mapa.cells[i][j+1] == '|' || (j != 127 && mapa.cells[i][j+1] == '-' && 
				   (i != 0 && mapa.cells[i-1][j+1] == '|' || i != 127 && mapa.cells[i+1][j+1] == '|')))
					k_stop = (int) ((lpr_v + 1) * 1.5);
				
				if (i != 127 && mapa.cells[i+1][j] == '-' ||
				   (i != 127 && mapa.cells[i+1][j] == '|' && j != 127 && mapa.cells[i+1][j+1] == '-'))		
					l_start = 1 - (lpr_v + 2)/2;
				
				if (i != 0 && mapa.cells[i-1][j] == '-' ||
				   (i != 0 && mapa.cells[i-1][j] == '|' && j != 127 && mapa.cells[i-1][j+1] == '-'))
					l_stop = (int) ((lpr_v + 2) * 1.5);
				
				vertexList = (vertex_t *) malloc(((k_stop - k_start + 2) * (l_stop - l_start + 1)) * sizeof(vertex_t));
				nr = 0;

				//generate all vertices for this wall
				for (l = l_start; l <= l_stop; l++) 
					for (k = k_start; k <= k_stop+1; k++) {
						vertexList[nr].x = width * (j + (float)(k-1)/(lpr_v+1)) ;
						vertexList[nr].y = height;
						vertexList[nr].z = width * (i + (float)(lpr_v + 2 - l) / (lpr_v + 1));
						if (l != l_start && l != l_stop && k != k_start && k != k_stop +1)
							vertexList[nr] = distortVertex(vertexList[nr], max_distort_val);
						nr++;
					}

				//create polygons from generated vertices
				for (k = 1; k <= k_stop - k_start + 1; k++)
					for (l = 1; l <= l_stop - l_start; l++) {
						v1 = vertexList[(l - 1) * (k_stop - k_start + 2) + k - 1];
						v2 = vertexList[(l - 1) * (k_stop - k_start + 2) + k    ];
						v3 = vertexList[ l      * (k_stop - k_start + 2) + k - 1];

						writePolygon(&file, v1, v2, v3); 
						number_of_polygons++;
						
						v1 = v3;
					 	v3 = vertexList[ l      * (k_stop - k_start + 2) + k    ];
					 
						writePolygon(&file, v1, v2, v3); 
						number_of_polygons++;	
					}

				free (vertexList);
			}


			
		}
	}
	fseek (file, 0, SEEK_SET);
	fprintf(file, "beginmodel %ld", number_of_polygons);
	fclose (file);
}

int main(int argc, char **argv)
{
    	if (argc != 3) {
	    	printf ("\nWrong number of parameters!\n\nUsage:\n\t./map_generator input_file output_file\n");
		exit(-1);
	}
	generateMap (argv[1], argv[2]);
	
        return(0);
}
