#include "includes.h"

void normal_f (float v0x, float v0y, float v0z, float v1x, float v1y, float v1z, float v2x, float v2y, float v2z,
        vector_t *normal)
{
    vector_t a, b;

    // calculate the vectors A and B
    // note that v[3] is defined with counterclockwise winding in mind
    // a
    a.x = v0x - v1x;
    a.y = v0y - v1y;
    a.z = v0z - v1z;
    // b
    b.x = v1x - v2x;
    b.y = v1y - v2y;
    b.z = v1z - v2z;

    // calculate the cross product and place the resulting vector
    // into the address specified by vertex_t *normal
    normal->x = (a.y * b.z) - (a.z * b.y);
    normal->y = (a.z * b.x) - (a.x * b.z);
    normal->z = (a.x * b.y) - (a.y * b.x);

    // normalize
    normalize(normal);
}

void normal_v (vertex_t v0, vertex_t v1, vertex_t v2, vector_t *normal)
{
    vector_t a, b;

    // calculate the vectors A and B
    // note that v[3] is defined with counterclockwise winding in mind
    // a
    a.x = v0.x - v1.x;
    a.y = v0.y - v1.y;
    a.z = v0.z - v1.z;
    // b
    b.x = v1.x - v2.x;
    b.y = v1.y - v2.y;
    b.z = v1.z - v2.z;

    // calculate the cross product and place the resulting vector
    // into the address specified by vertex_t *normal
    normal->x = (a.y * b.z) - (a.z * b.y);
    normal->y = (a.z * b.x) - (a.x * b.z);
    normal->z = (a.x * b.y) - (a.y * b.x);

    // normalize
    normalize(normal);
}

void normalize (vector_t *v)
{
    // calculate the length of the vector
    float len = (float)(sqrt((v->x * v->x) + (v->y * v->y) + (v->z * v->z)));

    // avoid division by 0
    if (len == 0.0f)
        len = 1.0f;

    // reduce to unit size
    v->x /= len;
    v->y /= len;
    v->z /= len;
}

