class Vertex;
class VertexV;
class Polygon;
class Scan2D;
class Scan3D;
class Scene;

//--------------------------------------------------------------
//--------------------------VERTEX------------------------------
//--------------------------------------------------------------

class Vertex {
public:
    double x;
    double y;
    double z;

    Vertex (double xx = 0, double yy = 0, double zz = 0);
    
    void Init (double xx, double yy, double zz);
    double DistanceTo (Vertex &);

};

//--------------------------------------------------------------
//--------------------------POLYGON-----------------------------
//--------------------------------------------------------------

class Polygon {
public:
    Vertex a;
    Vertex b;
    Vertex c;

    Polygon ();
    Polygon (Vertex, Vertex, Vertex);

    void Init (Vertex, Vertex, Vertex);
    void WriteToFile (FILE *); // !! opened !! file handle
    int IsValid(void);
    static int IsValid(Vertex &, char, Vertex &, char, Vertex &, char);
};

//--------------------------------------------------------------
//--------------------------SCAN_2D-----------------------------
//--------------------------------------------------------------

class Scan2D {
public:
    Vertex *col;
    char *valid;  //not every point is valid (< 8 m)
    
    Vertex base;
    double roll;  //commonly 0 - robot does not roll :)
    double pitch; //PTZ angle in degrees
    double yaw;   //scanner direction angle in degrees
    int cols;     //number of columns (points) stored

    Scan2D ();
    Scan2D (const Scan2D &);
    ~Scan2D ();

    Scan2D& operator=(const Scan2D &);
    int GetScan (PlayerClient &, LaserProxy &, TruthProxy &, double); //returns number of valid points, -1 on error
    int GetScan (PlayerClient &, LaserProxy &, PositionProxy &, double); //returns number of valid points, -1 on error
};


//--------------------------------------------------------------
//--------------------------SCAN_3D-----------------------------
//--------------------------------------------------------------

class Scan3D {
public:
    Scan2D *row;
    int rows;
    int cols;
    int valid_points;
    int polygon_number;
    std::vector<Polygon> polygons;

    Scan3D();
    Scan3D (const Scan3D &); 
    ~Scan3D();

    Scan3D& operator=(const Scan3D &); 
    int GetScan(PlayerClient &, LaserProxy &, TruthProxy &, PtzProxy &);
    int GetScan(PlayerClient &, LaserProxy &, PositionProxy &, PtzProxy &);
    int CalculatePolygons (void);
};


//--------------------------------------------------------------
//---------------------------SCENE------------------------------
//--------------------------------------------------------------

class Scene {

public:
    std::vector<Scan3D> scans;
    int valid_points;
    int polygon_number;

    Scene();
    ~Scene();

    void AddScan(PlayerClient &, LaserProxy &, TruthProxy &, PtzProxy &);
    void AddScan(PlayerClient &, LaserProxy &, PositionProxy &, PtzProxy &);
    void CalculatePolygons (void);
    
    void WriteCones (char *); //Writes scanned points as 3D cones to the scene file
    void WritePolygons (char *); //Writes polygons from every Scan3D to the scene file
	    
};
