#include "includes.h"

int init_ptz (PtzProxy &ptz) {

    logI ("init_ptz: Subscribing PTZ interface...\n");
    if (ptz.ChangeAccess('a') != 0)
    {
      logF ("init_ptz: could not subscribe PTZ interface!\n");
      return -1;
    }

    return 0;
}

void destroy_ptz (PtzProxy &ptz) {
    logD ("destroy_ptz: Destroying PTZ interface...\n");
    ptz.Close();
    ptz.~PtzProxy();
}

void set_tilt (PtzProxy &ptz, double deg) {
    logD ("set_tilt: Setting tilt to %lf degrees...\n", deg);

  //  if (deg > 0) deg += 0.1;	// these lines are workaround for 
  //  else         deg -= 0.1;    // bug in player ptz implementation
    
    if ( ptz.SetCam(0, deg2rad(deg), 0) != 0) {
	logE ("set_tilt: could not set the PTZ tilt!!\n");
    }
}

double get_tilt (PlayerClient &client, PtzProxy &ptz) {
    double val_rad, val_deg;
    client.Read();
    val_rad = ptz.tilt;
    val_deg = rad2deg(val_rad);
    logSD ("get_tilt: Reading PTZ Tilt = %lf (rad) %lf (deg)\n", val_rad, val_deg);
    return val_deg;
}


int wait_for_tilt (PlayerClient &client, PtzProxy &ptz, double deg) {
    struct timeval start,stop;
   
    set_tilt (ptz, deg);
    logD ("wait_for_tilt: will wait for tilt = %lf\n", deg);
   
    gettimeofday(&start, NULL);
    while (1) {
	gettimeofday(&stop, NULL);

	if ( fabs (get_tilt(client, ptz) - deg) < TILT_ANGLE_ACCEPTABLE_ERROR) { //OK
	    
	    if  (ms_passed(start, stop) < TILT_ANGLE_WAIT_MIN_TIME) continue;
		
	    logD ("wait_for_tilt: tilt reached after %lf seconds\n", (double)(stop.tv_sec  - start.tv_sec) + 
		    0.000001 * (double)(stop.tv_usec - start.tv_usec));
	    return 0; 
	}
	
	sleep_ms(TILT_ANGLE_POLL_MS);
	
	if ( ms_passed(start, stop) > TILT_ANGLE_WAIT_TIMEOUT) { //TIMEOUT
	    logW("wait_for_tilt: TIMEOUT!\n");
	    return -1;
	}
    }
}


